package com.yfd.platform.config;

import com.baomidou.mybatisplus.annotation.DbType;
import com.baomidou.mybatisplus.extension.plugins.MybatisPlusInterceptor;
import com.baomidou.mybatisplus.extension.plugins.handler.TableNameHandler;
import com.baomidou.mybatisplus.extension.plugins.inner.DynamicTableNameInnerInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.PaginationInnerInterceptor;
import com.yfd.platform.utils.TableNameContextHolder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.HashMap;
import java.util.Map;

/******************************
 * 用途说明:
 * 作者姓名: pcj
 * 创建时间: 2022/10/24 10:50
 ******************************/
@Configuration
public class MybitsPlusConfig {

    @Bean
    public MybatisPlusInterceptor mybatisPlusInterceptor() {
        MybatisPlusInterceptor interceptor = new MybatisPlusInterceptor();

        // 创建动态表名拦截器
        DynamicTableNameInnerInterceptor dynamicInterceptor = new DynamicTableNameInnerInterceptor();

        // 创建表名处理器映射
        Map<String, TableNameHandler> handlerMap = new HashMap<>();

        // 为 ts_files 表添加处理器
        handlerMap.put("ts_files", new TableNameHandler() {
            @Override
            public String dynamicTableName(String sql, String tableName) {
                String taskCode = TableNameContextHolder.getTaskCode();
                if (taskCode == null || taskCode.isEmpty()) {
                    throw new IllegalArgumentException("任务编码不能为空");
                }
                return "ts_files_" + taskCode;
            }
        });

        // 设置处理器映射
        dynamicInterceptor.setTableNameHandlerMap(handlerMap);
        // 添加动态表名拦截器
        interceptor.addInnerInterceptor(dynamicInterceptor);
        // 添加分页拦截器
        interceptor.addInnerInterceptor(new PaginationInnerInterceptor(DbType.MYSQL));
        return interceptor;
    }
}




